import com.test.*;
import javax.xml.datatype.DatatypeConstants;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;
import java.io.ByteArrayOutputStream;
import java.io.BufferedInputStream;
import java.io.FileOutputStream;
import java.util.List;
import com.objsys.xbinder.runtime.*;

/**
 * This test program encodes a data record and writes the encoded result to an output file.
 */
public class Writer {

   private static DatatypeFactory dtFactory;

   static {
      try {
         dtFactory = DatatypeFactory.newInstance();
      }
      catch ( javax.xml.datatype.DatatypeConfigurationException dce ) 
      { throw new RuntimeException(dce); }
   }

   private static String outputFile = "message.xml";
   private static String charset = "UTF-8";     //default is UTF-8
   private static boolean trace = true;

   public static void main(String args[]) throws Exception 
   {
      //process command line to get the name of the output file
      //process command line to get the name of the input file
      for(int arg = 0; arg < args.length; arg++) {
         if ( args[arg].equals("-o")) {
           outputFile = args[++arg];
         }
         else if ( args[arg].equals("-notrace" ) ) trace = false;
         else {
            System.out.println("Usage: java Writer [-o <filename>] [-notrace]");
            System.out.println("  -o <filename> specifies the output file");
            System.out.println("  -notrace turns off echoing encoded data to stdout");
            System.exit(1);
         }
      }

      //create _CC object
      Alert_CC root = new Alert_CC();

      Alert_ELEM alert;

      alert = new Alert_ELEM();
      root.setAlert(alert);
      alert.setIdentifier("43b080713727");
      alert.setSender("hsas@dhs.gov");

      XMLGregorianCalendar sent;
      sent = dtFactory.newXMLGregorianCalendar(2003, 04, 02, 14, 39, 1, DatatypeConstants.FIELD_UNDEFINED, -300);
      alert.setSent(sent);
      alert.setStatus(Alert_ELEM_status.Actual);
      alert.setMsgType(Alert_ELEM_msgType.Alert);
      alert.setScope(Alert_ELEM_scope.Public);

      java.util.List<com.test.Alert_ELEM_info> info = alert.getInfo();
      Alert_ELEM_info infoN;

      infoN = new Alert_ELEM_info();
      info.add(infoN);

      java.util.List<Alert_ELEM_info_category> category;
      category = infoN.getCategory();
      category.add(Alert_ELEM_info_category.Security);

      infoN.setEvent("Homeland Security Advisory System Update");
      infoN.setUrgency(Alert_ELEM_info_urgency.Immediate);
      infoN.setSeverity(Alert_ELEM_info_severity.Severe);
      infoN.setCertainty(Alert_ELEM_info_certainty.Likely);
      infoN.setSenderName("U.S. Government, Department of Homeland Security");
      infoN.setHeadline("Homeland Security Sets Code ORANGE");
      infoN.setDescription("The Department of Homeland Security has elevated the Homeland Security Advisory System threat level to ORANGE / High in response to intelligence which may indicate a heightened threat of terrorism.");
      infoN.setInstruction(" A High Condition is declared when there is a high risk of terrorist attacks. In addition to the Protective Measures taken in the previous Threat Conditions, Federal departments and agencies should consider agency specific Protective Measures in accordance with their existing plans.");
      infoN.setWeb("http://www.dhs.gov/dhspublic/display?theme=29");

      java.util.List<String> parameter;
      parameter = infoN.getParameter();
      parameter.add("HSAS=ORANGE");

      java.util.List<Alert_ELEM_info_resource> resource;
      resource = infoN.getResource();
      
      Alert_ELEM_info_resource resourceN = new Alert_ELEM_info_resource();
      resource.add(resourceN);
      resourceN.setResourceDesc("Image file (GIF)");
      resourceN.setUri("http://www.dhs.gov/dhspublic/getAdvisoryImage");
      
      java.util.List<Alert_ELEM_info_area> area = infoN.getArea();
      Alert_ELEM_info_area areaN = new Alert_ELEM_info_area();
      area.add(areaN);
      areaN.setAreaDesc("U.S. nationwide and interests worldwide");

      //Encode data
      ByteArrayOutputStream bos = new ByteArrayOutputStream(32768);

      XBXmlEncoder encoder;
      encoder = new XBXmlEncoder(bos, charset);
      root.encodeDocument(encoder);
      encoder.close();

      if ( trace ) {
         //write encoded data to System.out
         System.out.println("Encoded data:\n");
         bos.writeTo(System.out);
         System.out.println();
      }

      //write encoded data to output file
      FileOutputStream fout = new FileOutputStream(outputFile);
      bos.writeTo(fout);
      fout.close();
   }
}
