import com.test.*;
import javax.xml.datatype.DatatypeConstants;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;
import java.io.ByteArrayOutputStream;
import java.io.BufferedInputStream;
import java.io.FileOutputStream;
import java.util.List;
import com.objsys.xbinder.runtime.*;

/**
 * This test program encodes a data record and writes the encoded result to an output file.
 */
public class Writer {

   private static DatatypeFactory dtFactory;

   static {
      try {
         dtFactory = DatatypeFactory.newInstance();
      }
      catch ( javax.xml.datatype.DatatypeConfigurationException dce )
      { throw new RuntimeException(dce); }
   }

   private static String outputFile = "message.xml";
   private static String charset = "UTF-8";     //default is UTF-8
   private static boolean trace = true;
   private static final Empty EMPTY = new Empty();

   public static void main(String args[]) throws Exception
   {
      //process command line to get the name of the output file
      //process command line to get the name of the input file
      for(int arg = 0; arg < args.length; arg++) {
         if ( args[arg].equals("-o")) {
           outputFile = args[++arg];
         }
         else if ( args[arg].equals("-notrace" ) ) trace = false;
         else {
            System.out.println("Usage: java Writer [-o <filename>] [-notrace]");
            System.out.println("  -o <filename> specifies the output file");
            System.out.println("  -notrace turns off echoing encoded data to stdout");
            System.exit(1);
         }
      }

      //create _CC object
      FpML_CC root = new FpML_CC();

      //Populate structures
      DataDocument fpML = new DataDocument();
      root.setFpML(fpML);

      fpML.addNamespace("http://www.fpml.org/2007/FpML-4-4", "");
      fpML.addNamespace("http://www.w3.org/2000/09/xmldsig#", "dsig");
      fpML.addNamespace("http://www.eclipse.org/emf/2002/Ecore", "ecore");
      fpML.addNamespace("http://www.fpml.org/annotation", "fpml-annotation");

      fpML.setVersion( VersionAttributes_atts_version.x4_4 );

      DataDocument_4 dataDocument_4 = new DataDocument_4();
      fpML.setDataDocument_4( dataDocument_4 );

      DataDocument_5 dataDocument_5 = new DataDocument_5();
      dataDocument_4.setDataDocument_5( dataDocument_5 );

      java.util.List<Trade> trade;
      Trade tradeN;

      trade = dataDocument_5.getTrade();
      tradeN = new Trade();
      trade.add(tradeN);

      TradeHeader tradeHeader = new TradeHeader();
      tradeN.setTradeHeader(tradeHeader);

      java.util.List<PartyTradeIdentifier> partyTradeIdentifier;
      PartyTradeIdentifier partyTradeIdentifierN;

      partyTradeIdentifier = tradeHeader.getPartyTradeIdentifier();

      partyTradeIdentifierN = new PartyTradeIdentifier();
      partyTradeIdentifier.add(partyTradeIdentifierN);

      PartyReference partyReference = new PartyReference();
      partyTradeIdentifierN.setPartyReference( partyReference );
      partyReference.setHref("party1");

      java.util.List<TradeIdentifier_2> tradeIdentifier_2;
      TradeIdentifier_2 tradeIdentifier_2N;

      tradeIdentifier_2 = partyTradeIdentifierN.getTradeIdentifier_2();
      tradeIdentifier_2N = new TradeIdentifier_2();
      tradeIdentifier_2.add( tradeIdentifier_2N );

      TradeId tradeId = new TradeId();
      tradeIdentifier_2N.setTradeId( tradeId );
      tradeId.setTradeIdScheme( "http://www.swapswire.com/spec/2001/trade-id-1-0" );
      tradeId.setValue( "37209" );


      partyTradeIdentifierN = new PartyTradeIdentifier();
      partyTradeIdentifier.add(partyTradeIdentifierN);

      partyReference = new PartyReference();
      partyTradeIdentifierN.setPartyReference( partyReference );
      partyReference.setHref("party2");

      tradeIdentifier_2 = partyTradeIdentifierN.getTradeIdentifier_2();
      tradeIdentifier_2N = new TradeIdentifier_2();
      tradeIdentifier_2.add( tradeIdentifier_2N );

      tradeId = new TradeId();
      tradeIdentifier_2N.setTradeId( tradeId );
      tradeId.setTradeIdScheme( "http://www.swapswire.com/spec/2001/trade-id-1-0" );
      tradeId.setValue( "37209" );

      IdentifiedDate tradeDate;
      tradeDate = new IdentifiedDate();
      tradeHeader.setTradeDate(tradeDate);

      XMLGregorianCalendar value;
      value = dtFactory.newXMLGregorianCalendarDate(2002, 12, 04, 0);
      tradeDate.setValue( value );

      _productSG product = new _productSG();
      tradeN.setProduct(product);

      CreditDefaultSwap creditDefaultSwap = new CreditDefaultSwap();
      product.setCreditDefaultSwap( creditDefaultSwap );

      GeneralTerms generalTerms = new GeneralTerms();
      creditDefaultSwap.setGeneralTerms(generalTerms);

      AdjustableDate2 effectiveDate = new AdjustableDate2();
      generalTerms.setEffectiveDate( effectiveDate );

      IdentifiedDate unadjustedDate;
      unadjustedDate = new IdentifiedDate();
      effectiveDate.setUnadjustedDate( unadjustedDate );

      value = dtFactory.newXMLGregorianCalendarDate(2002, 12, 05, 0);
      unadjustedDate.setValue( value );

      AdjustableDate2_2 adjustableDate2_2 = new AdjustableDate2_2();
      effectiveDate.setAdjustableDate2_2( adjustableDate2_2 );

      BusinessDayAdjustments dateAdjustments = new BusinessDayAdjustments();
      adjustableDate2_2.setDateAdjustments( dateAdjustments );

      dateAdjustments.setBusinessDayConvention( BusinessDayConventionEnum.NONE );

      DeprecatedScheduledTerminationDate scheduledTerminationDate = new DeprecatedScheduledTerminationDate();
      generalTerms.setScheduledTerminationDate( scheduledTerminationDate );

      AdjustableDate2 adjustableDate = new AdjustableDate2();
      scheduledTerminationDate.setAdjustableDate( adjustableDate );

      unadjustedDate = new IdentifiedDate();
      adjustableDate.setUnadjustedDate( unadjustedDate );

      value = dtFactory.newXMLGregorianCalendarDate(2007, 12, 05, 0);
      unadjustedDate.setValue( value );

      adjustableDate2_2 = new AdjustableDate2_2();
      adjustableDate.setAdjustableDate2_2( adjustableDate2_2 );

      dateAdjustments = new BusinessDayAdjustments();
      adjustableDate2_2.setDateAdjustments( dateAdjustments );

      dateAdjustments.setBusinessDayConvention( BusinessDayConventionEnum.MODFOLLOWING );

      BusinessCentersOrReference_model_Group businessCentersOrReference_model;
      businessCentersOrReference_model = new BusinessCentersOrReference_model_Group();
      dateAdjustments.setBusinessCentersOrReference_model( businessCentersOrReference_model );

      BusinessCenters businessCenters = new BusinessCenters();
      businessCentersOrReference_model.setBusinessCenters( businessCenters );

      java.util.List<BusinessCenter> businessCenter;
      BusinessCenter businessCenterN;

      businessCenter = businessCenters.getBusinessCenter();

      businessCenterN = new BusinessCenter();
      businessCenter.add( businessCenterN );

      //businessCenterN.setBusinessCenterScheme("http://www.fpml.org/coding-scheme/business-center-6-5");
      businessCenterN.setValue("GBLO");

      businessCenterN = new BusinessCenter();
      businessCenter.add( businessCenterN );

      //businessCenterN.setBusinessCenterScheme("http://www.fpml.org/coding-scheme/business-center-6-5");
      businessCenterN.setValue("USNY");


      businessCenterN = new BusinessCenter();
      businessCenter.add( businessCenterN );

      //businessCenterN.setBusinessCenterScheme("http://www.fpml.org/coding-scheme/business-center-6-5");
      businessCenterN.setValue("JPTO");

      PartyOrTradeSideReference sellerPartyReference = new PartyOrTradeSideReference();
      generalTerms.setSellerPartyReference( sellerPartyReference );
      sellerPartyReference.setHref("party1");

      PartyOrTradeSideReference buyerPartyReference = new PartyOrTradeSideReference();
      generalTerms.setBuyerPartyReference( buyerPartyReference );
      buyerPartyReference.setHref("party2");

      dateAdjustments = new BusinessDayAdjustments();
      generalTerms.setDateAdjustments( dateAdjustments );

      dateAdjustments.setBusinessDayConvention( BusinessDayConventionEnum.MODFOLLOWING );

      businessCentersOrReference_model = new BusinessCentersOrReference_model_Group();
      dateAdjustments.setBusinessCentersOrReference_model( businessCentersOrReference_model );

      businessCenters = new BusinessCenters();
      businessCentersOrReference_model.setBusinessCenters( businessCenters );

      businessCenter = businessCenters.getBusinessCenter();

      businessCenterN = new BusinessCenter();
      businessCenter.add( businessCenterN );

      //businessCenterN.setBusinessCenterScheme("http://www.fpml.org/coding-scheme/business-center-6-5");
      businessCenterN.setValue("GBLO");

      businessCenterN = new BusinessCenter();
      businessCenter.add( businessCenterN );

      //businessCenterN.setBusinessCenterScheme("http://www.fpml.org/coding-scheme/business-center-6-5");
      businessCenterN.setValue("USNY");


      businessCenterN = new BusinessCenter();
      businessCenter.add( businessCenterN );

      //businessCenterN.setBusinessCenterScheme("http://www.fpml.org/coding-scheme/business-center-6-5");
      businessCenterN.setValue("JPTO");

      GeneralTerms_6 generalTerms_6 = new GeneralTerms_6();
      generalTerms.setGeneralTerms_6( generalTerms_6 );

      ReferenceInformation referenceInformation = new ReferenceInformation();
      generalTerms_6.setReferenceInformation( referenceInformation );

      LegalEntity referenceEntity = new LegalEntity();
      referenceInformation.setReferenceEntity( referenceEntity );

      referenceEntity.setId("referenceEntity");
      LegalEntity_1 legalEntity_1 = new LegalEntity_1();
      referenceEntity.setLegalEntity_1( legalEntity_1 );

      EntityName entityName = new EntityName();
      legalEntity_1.setEntityName( entityName );
      //entityName.setEntityNameScheme("http://www.fpml.org/spec/2003/entity-name-RED-1-0");
      entityName.setValue("ACOM CO., LTD.");

      java.util.List<EntityId> entityId;
      entityId = legalEntity_1.getEntityId();
      EntityId entityIdN = new EntityId();
      entityId.add( entityIdN );
      entityIdN.setEntityIdScheme("http://www.fpml.org/spec/2003/entity-id-RED-1-0");
      entityIdN.setValue("004CC9");

      ReferenceInformation_2 referenceInformation_2 = new ReferenceInformation_2();
      referenceInformation.setReferenceInformation_2( referenceInformation_2 );

      java.util.List<com.test.ReferenceObligation> referenceObligation;
      ReferenceObligation referenceObligationN;

      referenceObligation = referenceInformation_2.getReferenceObligation();
      referenceObligationN = new ReferenceObligation();
      referenceObligation.add( referenceObligationN );

      ReferenceObligation_1 referenceObligation_1 = new ReferenceObligation_1();
      referenceObligationN.setReferenceObligation_1( referenceObligation_1 );

      Bond bond = new Bond();
      referenceObligation_1.setBond( bond );

      java.util.List<InstrumentId> instrumentId = bond.getInstrumentId();
      InstrumentId instrumentIdN = new InstrumentId();
      instrumentId.add( instrumentIdN );

      instrumentIdN.setInstrumentIdScheme("http://www.fpml.org/spec/2002/instrument-id-ISIN-1-0");
      instrumentIdN.setValue("JP310860A032");

      BondContent_model_Group bondContent_model = new BondContent_model_Group();
      bond.setBondContent_model( bondContent_model );

      bondContent_model.setCouponRate( new java.math.BigDecimal("0.0213") );
      XMLGregorianCalendar maturity = dtFactory.newXMLGregorianCalendarDate(2007, 3, 8, DatatypeConstants.FIELD_UNDEFINED);
      bondContent_model.setMaturity( maturity );

      ReferenceObligation_6 referenceObligation_6 = new ReferenceObligation_6();
      referenceObligationN.setReferenceObligation_6( referenceObligation_6 );

      LegalEntityReference primaryObligorReference = new LegalEntityReference();
      referenceObligation_6.setPrimaryObligorReference( primaryObligorReference );
      primaryObligorReference.setHref("referenceEntity");

      referenceInformation.setReferencePrice( new java.math.BigDecimal("1.0") );

      FeeLeg feeLeg = new FeeLeg();
      creditDefaultSwap.setFeeLeg( feeLeg );

      PeriodicPayment periodicPayment = new PeriodicPayment();
      feeLeg.setPeriodicPayment( periodicPayment );

      Interval paymentFrequency = new Interval();
      periodicPayment.setPaymentFrequency( paymentFrequency );

      paymentFrequency.setPeriodMultiplier(3);
      paymentFrequency.setPeriod(PeriodEnum.M);

      XMLGregorianCalendar firstPaymentDate = dtFactory.newXMLGregorianCalendarDate(2003, 03, 05, 0);
      periodicPayment.setFirstPaymentDate( firstPaymentDate );
      periodicPayment.setRollConvention( RollConventionEnum.x5 );

      PeriodicPayment_6 periodicPayment_6 = new PeriodicPayment_6();
      periodicPayment.setPeriodicPayment_6( periodicPayment_6 );

      FixedAmountCalculation fixedAmountCalculation = new FixedAmountCalculation();
      periodicPayment_6.setFixedAmountCalculation( fixedAmountCalculation );

      CalculationAmount calculationAmount = new CalculationAmount();
      fixedAmountCalculation.setCalculationAmount( calculationAmount );

      Currency currency = new Currency();
      calculationAmount.setCurrency( currency );

      //currency.setCurrencyScheme("http://www.fpml.org/ext/iso4217-2001-08-15");
      currency.setValue("JPY");

      calculationAmount.setAmount( new java.math.BigDecimal("500000000.0"));

      FixedRate fixedRate = new FixedRate();
      fixedAmountCalculation.setFixedRate(fixedRate);
      fixedRate.setValue( new java.math.BigDecimal("0.007"));

      DayCountFraction dayCountFraction = new DayCountFraction() ;
      fixedAmountCalculation.setDayCountFraction(dayCountFraction);

      //dayCountFraction.setDayCountFractionScheme("http://www.fpml.org/coding-scheme/day-count-fraction-2-1");
      dayCountFraction.setValue("ACT/360");

      java.util.List<ProtectionTerms> protectionTerms;
      ProtectionTerms protectionTermsN;

      protectionTerms = creditDefaultSwap.getProtectionTerms();
      protectionTermsN = new ProtectionTerms();
      protectionTerms.add(protectionTermsN);

      Money ptCalculationAmount = new Money();
      protectionTermsN.setCalculationAmount( ptCalculationAmount );

      currency = new Currency();
      ptCalculationAmount.setCurrency( currency );

      //currency.setCurrencyScheme("http://www.fpml.org/ext/iso4217-2001-08-15");
      currency.setValue("JPY");

      ptCalculationAmount.setAmount(new java.math.BigDecimal("500000000.0") );

      CreditEvents creditEvents = new CreditEvents();
      protectionTermsN.setCreditEvents( creditEvents );

      creditEvents.setBankruptcy( EMPTY );
      FailureToPay failureToPay = new FailureToPay();
      creditEvents.setFailureToPay( failureToPay );

      Money paymentRequirement = new Money();
      failureToPay.setPaymentRequirement( paymentRequirement );

      currency = new Currency();
      paymentRequirement.setCurrency( currency );

      //currency.setCurrencyScheme("http://www.fpml.org/ext/iso4217-2001-08-15");
      currency.setValue("JPY");

      paymentRequirement.setAmount(new java.math.BigDecimal("100000000.0") );

      Restructuring restructuring = new Restructuring();
      creditEvents.setRestructuring( restructuring );

      RestructuringType restructuringType = new RestructuringType();
      restructuring.setRestructuringType( restructuringType );

      //restructuringType.setRestructuringScheme("http://www.fpml.org/coding-scheme/restructuring-1-0");
      restructuringType.setValue("R");

      Money defaultRequirement = new Money();
      creditEvents.setDefaultRequirement( defaultRequirement );

      currency = new Currency();
      defaultRequirement.setCurrency( currency );

      //currency.setCurrencyScheme("http://www.fpml.org/ext/iso4217-2001-08-15");
      currency.setValue("JPY");

      defaultRequirement.setAmount(new java.math.BigDecimal("1000000000.0") );

      CreditEventNotice creditEventNotice = new CreditEventNotice();
      creditEvents.setCreditEventNotice( creditEventNotice );

      NotifyingParty notifyingParty = new NotifyingParty();
      creditEventNotice.setNotifyingParty( notifyingParty );

      PartyReference ceBuyerPartyReference = new PartyReference();
      notifyingParty.setBuyerPartyReference( ceBuyerPartyReference );
      ceBuyerPartyReference.setHref("party2");

      PartyReference ceSellerPartyReference = new PartyReference();
      notifyingParty.setSellerPartyReference( ceSellerPartyReference );
      ceSellerPartyReference.setHref("party1");

      PubliclyAvailableInformation publiclyAvailableInformation = new PubliclyAvailableInformation();
      creditEventNotice.setPubliclyAvailableInformation( publiclyAvailableInformation );

      publiclyAvailableInformation.setStandardPublicSources( EMPTY );
      publiclyAvailableInformation.setSpecifiedNumber(2);

      Obligations obligations = new Obligations();
      protectionTermsN.setObligations( obligations );

      obligations.setCategory( ObligationCategoryEnum.BorrowedMoney );
      obligations.setNotSubordinated(EMPTY);

      java.util.List<CreditDefaultSwap_6> creditDefaultSwap_6;
      CreditDefaultSwap_6 creditDefaultSwap_6N;

      creditDefaultSwap_6 = creditDefaultSwap.getCreditDefaultSwap_6();

      creditDefaultSwap_6N = new CreditDefaultSwap_6();
      creditDefaultSwap_6.add( creditDefaultSwap_6N );

      PhysicalSettlementTerms physicalSettlementTerms = new PhysicalSettlementTerms();
      creditDefaultSwap_6N.setPhysicalSettlementTerms(physicalSettlementTerms);

      currency = new Currency();
      physicalSettlementTerms.setSettlementCurrency( currency );

      //currency.setCurrencyScheme("http://www.fpml.org/ext/iso4217-2001-08-15");
      currency.setValue("JPY");

      PhysicalSettlementPeriod physicalSettlementPeriod = new PhysicalSettlementPeriod();
      physicalSettlementTerms.setPhysicalSettlementPeriod( physicalSettlementPeriod );
      physicalSettlementPeriod.setBusinessDays(30);

      DeliverableObligations deliverableObligations = new DeliverableObligations();
      physicalSettlementTerms.setDeliverableObligations( deliverableObligations );

      deliverableObligations.setAccruedInterest(false);
      deliverableObligations.setCategory( ObligationCategoryEnum.BondOrLoan );
      deliverableObligations.setNotSubordinated( EMPTY );
      deliverableObligations.setSpecifiedCurrency( new SpecifiedCurrency() );
      deliverableObligations.setNotContingent( EMPTY );
      deliverableObligations.setAssignableLoan( new PCDeliverableObligationCharac()  );
      deliverableObligations.setConsentRequiredLoan( new PCDeliverableObligationCharac() );
      deliverableObligations.setTransferable( EMPTY );

      Interval maximumMaturity = new Interval();
      deliverableObligations.setMaximumMaturity( maximumMaturity );
      maximumMaturity.setPeriodMultiplier(30);
      maximumMaturity.setPeriod( PeriodEnum.Y );

      deliverableObligations.setNotBearer( EMPTY );

      physicalSettlementTerms.setEscrow(true);

      CalculationAgent_model_Group calculationAgent_model = new CalculationAgent_model_Group();
      tradeN.setCalculationAgent_model( calculationAgent_model );

      CalculationAgent calculationAgent = new CalculationAgent();
      calculationAgent_model.setCalculationAgent( calculationAgent );

      java.util.List<PartyReference> calculationAgentPartyReference = calculationAgent.getCalculationAgentPartyReference();
      partyReference = new PartyReference();
      calculationAgentPartyReference.add( partyReference );
      partyReference.setHref("party1");

      BusinessCenter calculationAgentBusinessCenter = new BusinessCenter();
      calculationAgent_model.setCalculationAgentBusinessCenter( calculationAgentBusinessCenter );

      //calculationAgentBusinessCenter.setBusinessCenterScheme("http://www.fpml.org/coding-scheme/business-center-6-5");
      calculationAgentBusinessCenter.setValue("GBLO");

      Documentation documentation = new Documentation();
      tradeN.setDocumentation(documentation);

      MasterAgreement masterAgreement = new MasterAgreement();
      documentation.setMasterAgreement( masterAgreement );

      MasterAgreementType masterAgreementType = new MasterAgreementType();
      masterAgreement.setMasterAgreementType( masterAgreementType );

      //masterAgreementType.setMasterAgreementTypeScheme("http://www.fpml.org/coding-scheme/master-agreement-type-1-0");
      masterAgreementType.setValue("ISDA1992");

      java.util.List<ContractualDefinitions> contractualDefinitions_List = documentation.getContractualDefinitions();
      ContractualDefinitions contractualDefinitions = new ContractualDefinitions();
      contractualDefinitions_List.add( contractualDefinitions );

      //contractualDefinitions.setContractualDefinitionsScheme("http://www.fpml.org/coding-scheme/contractual-definitions-3-2");
      contractualDefinitions.setValue("ISDA1999Credit");

      Documentation_6 documentation_6 = new Documentation_6();
      documentation.setDocumentation_6( documentation_6 );

      java.util.List<ContractualTermsSupplement> contractualTermsSupplement_List = documentation_6.getContractualTermsSupplement();
      ContractualTermsSupplement contractualTermsSupplement = new ContractualTermsSupplement();
      contractualTermsSupplement_List.add(contractualTermsSupplement);

      ContractualSupplement type = new ContractualSupplement();
      contractualTermsSupplement.setType( type );

      //type.setContractualSupplementScheme("http://www.fpml.org/coding-scheme/contractual-supplement-6-8");
      type.setValue("ISDA1999CreditSuccessorAndCreditEvents");

      contractualTermsSupplement = new ContractualTermsSupplement();
      contractualTermsSupplement_List.add(contractualTermsSupplement);

      type = new ContractualSupplement();
      contractualTermsSupplement.setType( type );

      //type.setContractualSupplementScheme("http://www.fpml.org/coding-scheme/contractual-supplement-6-8");
      type.setValue("ISDA1999CreditConvertibleExchangeableAccretingObligations");

      java.util.List<Party> party_List = fpML.getParty();
      Party party = new Party();
      party_List.add(party);

      party.setId("party1");
      java.util.List<PartyId> partyId_List = party.getPartyId();
      PartyId partyId = new PartyId();
      partyId_List.add(partyId);

      //partyId.setPartyIdScheme("http://www.fpml.org/ext/iso9362");
      partyId.setValue("XYZBICXXX");

      party.setPartyName("XYZ Bank");

      party = new Party();
      party_List.add(party);

      party.setId("party2");
      partyId_List = party.getPartyId();
      partyId = new PartyId();
      partyId_List.add(partyId);

      //partyId.setPartyIdScheme("http://www.fpml.org/ext/iso9362");
      partyId.setValue("ABCBICXXX");

      party.setPartyName("ABC Bank");



      //Encode data
      ByteArrayOutputStream bos = new ByteArrayOutputStream(32768);

      XBXmlEncoder encoder;
      encoder = new XBXmlEncoder(bos, charset);

      root.encodeDocument(encoder);
      encoder.close();

      if ( trace ) {
         //write encoded data to System.out
         System.out.println("Encoded data:\n");
         bos.writeTo(System.out);
         System.out.println();
      }

      //write encoded data to output file
      FileOutputStream fout = new FileOutputStream(outputFile);
      bos.writeTo(fout);
      fout.close();
   }
}
